# DESCRIPTION:
# This is a TCL library that handles calling CryptoPhoto.
#    - Main Page
#        http://cryptophoto.com/
#    - About Cryptophoto
#         http://cryptophoto.com/about
#    - Register to CryptoPhoto
#        http://cryptophoto.com
#
# VERSION: 1.20170608
# COPYRIGHT(c) 2017 CryptoPhoto -- http://cryptophoto.com/
# AUTHOR: CryptoPhoto
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

package require sha1

namespace eval ::cryptophotoutils {
    namespace export reset_vars start_session get_gen_widget get_auth_widget verify_response

    variable pubkey ""
    variable privkey ""
    variable salt ""
    variable server "https://cryptophoto.com"
    variable uid ""
    variable sid ""
    variable SERVER_ERROR "error-server-not-reachable"
    variable REQUEST_ERROR "error-bad-request"
}

proc ::cryptophotoutils::reset_vars {parms} {
    variable pubkey
    variable privkey
    variable salt
    variable uid
    variable server
    variable sid
    
    set sid ""
    set pubkey [dict get $parms pubkey]
    set privkey [dict get $parms privkey]
    set uid [dict get $parms uid]
    set salt [dict get $parms salt]
    if { [dict get $parms server] ne "" } {
        set server [dict get $parms server]
    }
}

proc ::cryptophotoutils::string_encode {data} {
    set encodedstr ""
    dict for {k v} $data {
        append encodedstr "--data \"${k}=${v}\" "
    }
    return $encodedstr
}

proc ::cryptophotoutils::post_request {url data} {
    set curl_parms [cryptophotoutils::string_encode $data]
    
    set cmd "/tmp/customCURLwebui -s --request POST $curl_parms $url"
    set output ""

    catch {
        set output [exec {*}$cmd]
    } err
    
    return $output
}

proc ::cryptophotoutils::start_session {ip authentication} {
    variable pubkey
    variable privkey
    variable salt
    variable uid
    variable server
    variable sid
    variable SERVER_ERROR
    
    set timestamp [clock seconds]
    set hashed_uid [sha1::sha1 "$salt$uid"]
    set signature [sha1::hmac $privkey "$privkey${timestamp}${hashed_uid}$pubkey"]
    set data [dict create publickey $pubkey uid $hashed_uid time $timestamp signature $signature ip $ip]
    
    if { $authentication eq "true" } {
        dict set data authentication "true"
    } else {
        dict set data authentication "false"
    }
    
    set response [cryptophotoutils::post_request "$server/api/get/session" $data]
    
    set records [split $response "\n"]
    
    if { [lindex $records 0] eq "success" } {
        set sid [lindex $records 1]
        return [dict create isValid 1 sid [lindex $records 1] hasToken [lindex $records 2]]
    } else {
        set res [dict create isValid 0 err [lindex $records 1]]
        
        if { [llength $records] == 4 } {
            dict set res errIP [lindex $records 3]
        }
        return $res
    }
    
    return [dic create isValid 0 err $SERVER_ERROR]
}


proc ::cryptophotoutils::get_gen_widget {} {
    variable server
    variable sid
    if { $sid ne "" } {
        return "<script type='text/javascript' src='${server}/api/token?sd=$sid' async></script>"
    } else {
        return "Generate session first"
    }
}

proc ::cryptophotoutils::get_auth_widget {} {
    variable server
    variable sid
    
    if { $sid ne "" } {
        return "<script type='text/javascript' src='${server}/api/challenge?sd=$sid' async></script>"
    } else {
        return "Generate session first"
    }
    
    
}

proc ::cryptophotoutils::verify_response {selector response_row response_col cp_phc ip} {
    variable pubkey
    variable privkey
    variable salt
    variable uid
    variable server
    variable SERVER_ERROR
    variable REQUEST_ERROR
    
    if { $selector eq "" } {
        return [dict create isValid 0 err $REQUEST_ERROR]
    } else {
        if {($response_row eq "" && $response_col eq "") && ($cp_phc eq "")} {
            return [dict create isValid 0 err $REQUEST_ERROR]
        }
    }
    
    set timestamp [clock seconds]
    set hashed_uid [sha1::sha1 "$salt$uid"]
    set signature [sha1::hmac $privkey "$privkey${timestamp}${hashed_uid}$pubkey"]
    
    set data [dict create publickey $pubkey uid $hashed_uid time $timestamp signature $signature ip $ip]
    
    if { $response_row ne "" } {
        dict set data response_row $response_row
    }
    if { $response_col ne "" } {
        dict set data response_col $response_col
    }
    if { $selector ne "" } {
        dict set data selector $selector
    }
    if { $cp_phc ne "" } {
        dict set data cph $cp_phc
    }
    
    set response [cryptophotoutils::post_request "$server/api/verify" $data]
    
    set records [split $response "\n"]
    
    if { [lindex $records 0] eq "success" } {
        return [dict create isValid 1 msg [lindex $records 1]]
    } else {
        return [dict create isValid 0 err [lindex $records 1]]
    }
    
    return [dic create isValid 0 err $SERVER_ERROR]
    
}

package provide cryptophotoutils 1.0.0
